/*
 *  shadowircd: an advanced Internet Relay Chat Daemon(ircd).
 *  svsjoin.c: Forces joins.
 *
 *  $Id: svsjoin.c,v 1.1.2.4 2004/10/24 00:36:58 nenolod Exp $
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "ircd.h"
#include "numeric.h"
#include "sprintf_irc.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "hash.h"
#include "modules.h"
#include "s_conf.h"
#include "channel.h"
#include "channel_mode.h"
#include "common.h"
#include "irc_string.h"
#include "list.h"
#include "numeric.h"
#include "s_user.h"
#include "whowas.h"
#include "event.h"
#include "memory.h"
#include "balloc.h"
#include "resv.h"

static void m_svsjoin (struct Client *, struct Client *, int, char **);

struct Message svsjoin_msgtab = {
  "SVSJOIN", 0, 0, 0, 0, MFLG_SLOW | MFLG_UNREG, 0,
  {m_unregistered, m_ignore, m_svsjoin, m_ignore, m_ignore}
};

#ifndef STATIC_MODULES
void
_modinit (void)
{
  mod_add_cmd (&svsjoin_msgtab);
}

void
_moddeinit (void)
{
  mod_del_cmd (&svsjoin_msgtab);
}

const char *_version = "$Revision: 1.1.2.4 $";
#endif

/*
** m_svsjoin
**      parv[0] = sender prefix
**      parv[1] = victim
**	parv[2] = channel
*/
static void
m_svsjoin (struct Client *client_p, struct Client *source_p,
	   int parc, char *parv[])
{
  struct Client *target_p;
  struct Channel *chptr;
  int flags;

  if (!IsServer (source_p))
    return;

  if ((target_p = (struct Client *) find_client (parv[1])) == NULL)
    return;

  chptr = get_or_create_channel (target_p, parv[2], &flags);

  if (!IsMember (target_p, chptr))
    {
      add_user_to_channel (chptr, target_p, 0);

      if (flags & CHFL_CHANOP)
	{
	  chptr->channelts = CurrentTime;
	  chptr->mode.mode |= MODE_TOPICLIMIT;
	  chptr->mode.mode |= MODE_NOPRIVMSGS;

	  sendto_channel_local (ALL_MEMBERS, chptr,
				":%s MODE %s +nt", me.name, chptr->chname);
	}

      if (chptr->chname[0] != '&')
	{
	  sendto_server (NULL, PROTO_CUR, PROTO_CUR,
			 ":%s SJOIN %lu %s +nt :%s%s", me.id,
			 (unsigned long) chptr->channelts, chptr->chname,
			 (flags & CHFL_CHANOP) ? "@" : "", target_p->id);
	  sendto_server (NULL, PROTO_NUL, PROTO_MIN,
			 ":%s SJOIN %lu %s +nt :%s%s", me.name,
			 (unsigned long) chptr->channelts, chptr->chname,
			 (flags & CHFL_CHANOP) ? "@" : "", target_p->name);
	}

      sendto_channel_local (ALL_MEMBERS, chptr, ":%s!%s@%s JOIN :%s",
			    target_p->name, target_p->username,
			    GET_CLIENT_HOST (target_p), chptr->chname);

      if (IsOper (source_p) && ConfigFileEntry.oper_prefix)
	sendto_channel_local (ALL_MEMBERS, chptr, ":%s MODE %s +a %s",
			      me.name, chptr->chname, target_p->name);

      if (MyClient (target_p))
	{
	  if (chptr->topic != NULL)
	    {
	      sendto_one (target_p, form_str (RPL_TOPIC), me.name,
			  target_p->name, chptr->chname, chptr->topic);
	      sendto_one (target_p, form_str (RPL_TOPICWHOTIME),
			  me.name, target_p->name, chptr->chname,
			  chptr->topic_info, chptr->topic_time);
	    }
	  target_p->localClient->last_join_time = CurrentTime;
	  channel_member_names (source_p, chptr, 1);
	}
      sendto_server (NULL, PROTO_CUR, PROTO_CUR,
		     ":%s SVSJOIN %s %s", me.id, parv[1], parv[2]);
      sendto_server (NULL, PROTO_NUL, PROTO_MIN,
		     ":%s SVSJOIN %s %s", me.name, parv[1], parv[2]);
    }
}
